// PSound (CPS1 sound)
#include "cps.h"

static int nZExtra;										// Extra Z80 cycles we did last time

static int nDid;										// Z80 cycles actually executed thsi frame

unsigned char PsndCode, PsndFade;						// Sound code/fade sent to the z80 program

int PsndInit()
{
	nCpsZ80Cycles = 4000000 * 100 / nBurnFPS;

	// Init PSound z80
	if (PsndZInit()!= 0) {
		return 1;
	}

	// Init PSound mixing (not critical if it fails)
	PsmInit();

	nZExtra = 0;
	PsndCode = 0; PsndFade = 0;

	return 0;
}

int PsndExit()
{
	PsmExit();
	PsndZExit();

	return 0;
}

int PsndScan(int nAction)
{
	SCAN_VAR(nZExtra);
	PsndZScan(nAction);							// Scan Z80
	SCAN_VAR(PsndCode); SCAN_VAR(PsndFade);		// Scan sound info

	return 0;
}

// Execute the z80 for a section
int PsndSectRun(int nStart, int nEnd)
{
	int nAtEnd, nAtStart, nWant = 0;

	if (nEnd <= nStart) {
		return 1;
	}

	nAtStart = (nCpsZ80Cycles * nStart) >> 10;
	nAtEnd   = (nCpsZ80Cycles * nEnd  ) >> 10;

	nDid = 0;

	// Cycles would we like to do this time to get to end point
	nWant = nAtEnd - nAtStart - nZExtra;

	if (nWant > 0) {
		nDid += PsndZRun(nWant);
	}

	PsmSect(nStart, nEnd);						// create sound

	nZExtra = nDid - nWant;

	return 0;
}
