#include "cps.h"
#include "burn_ym2151.h"

// CPS1 sound Mixing

int bPsmOkay = 0;										// 1 if the module is okay
static short* WaveBuf=NULL;

int PsmInit()
{
	int nMemLen, nRate, nRet;
	bPsmOkay = 0;										// not OK yet

	if (nBurnSoundRate > 0) {
		nRate = nBurnSoundRate;
	} else {
		nRate = 11025;
	}

	if (BurnYM2151Init(3579540, 50.0)) {				// Init FM sound chip
		return 1;
	}

	// Allocate a buffer for the intermediate sound (between YM2151 and pBurnSoundOut)
	nMemLen = nBurnSoundLen * 2 * sizeof(short);
	WaveBuf = (short*)malloc(nMemLen);
	if (WaveBuf == NULL) {
		PsmExit();
		return 1;
	}
	memset(WaveBuf, 0, nMemLen);						// Init to silence

	// Init ADPCM
	MSM6295ROM = CpsAd;
	if (Forgottn) {
		nRet = MSM6295Init(0, 6061, 21.5, 1);
	} else {
		nRet = MSM6295Init(0, 7576, 21.5, 1);
	}

	if (nRet!=0) {
		PsmExit(); return 1;
	}

	bPsmOkay = 1;										// OK

	return 0;
}

int PsmExit()
{
	bPsmOkay = 0;

	MSM6295Exit(0);

	free(WaveBuf);
	WaveBuf = NULL;

	BurnYM2151Exit();									// Exit FM sound chip
	return 0;
}

// Render a section of sound
// e.g. 0x200 to 0x400 for second half of frame
int PsmSect(int nStart, int nEnd)
{
	int p1, p2;

	if (bPsmOkay == 0 || pBurnSoundOut == NULL) {
		return 1;
	}

	if (nEnd <= nStart) {
		return 0;
	}

	// Copy this portion of YM2151 sound into our WaveBuf buffer
	p1 = (nStart * nBurnSoundLen) >> 10;
	p2 = (nEnd   * nBurnSoundLen) >> 10;

	// Render FM
	BurnYM2151Render(pBurnSoundOut + (p1 << 1), p2 - p1);

	// Render ADPCM
	MSM6295Render(0, pBurnSoundOut + (p1 << 1), p2 - p1);

	return 0;
}
