#include "cps.h"
// QSound

static int nZExtra;						// Extra Z80 cycles we did last time
static int nIrqPeriod;					// period of IRQ in frames << 10
										// (because of the 0-0x400 frame ticks)
static int nIrqNext;

// QSound
int QsndInit()
{
	int nVolumeShift;
	int nRate;

	if (Cps1Qs == 1) {
		nCpsZ80Cycles = 6000000 * 100 / nBurnFPS;
	} else {
		nCpsZ80Cycles = 8000000 * 100 / nBurnFPS;
	}

	nIrqPeriod = (nBurnFPS << 10) / (252 * 100);

	// Init QSound z80
	if (QsndZInit()) {
		return 1;
	}

	nZExtra = 0;
	nIrqNext = nIrqPeriod;

	if (nBurnSoundRate >= 0) {
		nRate = nBurnSoundRate;
	} else {
		nRate = 11025;
	}

	nVolumeShift = 0;

	// These games are too soft at normal volumes
	if (strncmp(BurnDrvText(0), "csclub", 6) == 0) {
		nVolumeShift = -1;
	}
#if 0
	// These games are loud at normal volumes (no clipping)
	if (strncmp(BurnDrvText(0), "1944",	  4) == 0 ||
		strcmp( BurnDrvText(0), "dimahoo"  ) == 0 ||
		strcmp( BurnDrvText(0), "gmahoo"   ) == 0)
	{
		nVolumeShift = 1;
	}
#endif
	// These games are too loud at normal volumes (no clipping)
	if (strncmp(BurnDrvText(0), "sgemf",  5) == 0 ||
		strncmp(BurnDrvText(0), "pfght",  5) == 0 ||
		strncmp(BurnDrvText(0), "mpang",  5) == 0 ||
		strncmp(BurnDrvText(0), "spf2",   4) == 0 ||
		strncmp(BurnDrvText(0), "sfa2",   4) == 0 ||
		strncmp(BurnDrvText(0), "sfa2",   4) == 0)
	{
		nVolumeShift = 1;
	}
	// These games are too loud at normal volumes (clipping)
	if (strncmp(BurnDrvText(0), "19xx",   4) == 0 ||
		strncmp(BurnDrvText(0), "ddtod",  5) == 0)
	{
		nVolumeShift = 2;
	}

	QscInit(nRate, nVolumeShift);		// Init QSound chip

	return 0;
}

int QsndExit()
{
	QscExit();							// Exit QSound chip
	QsndZExit();
	return 0;
}

int QsndScan(int nAction)
{
	SCAN_VAR(nZExtra);					// Scan extra cycles
	SCAN_VAR(nIrqNext);					// Scan next irq time
	QsndZScan(nAction);					// Scan Z80
	QscScan(nAction);					// Scan QSound Chip

	return 0;
}

// Render sound for a section of one frame,
// e.g. 0x200 to 0x400 for second half
inline static void SectSound(int nStart,int nEnd)
{
	int nAtEnd, nAtStart;
	short *Dest; int nSam;
	if (pBurnSoundOut == NULL) {
		return;
	}

	nAtStart = (nBurnSoundLen * nStart) >> 10;
	nAtEnd   = (nBurnSoundLen * nEnd  ) >> 10;

	Dest = pBurnSoundOut + (nAtStart << 1);
	nSam = nAtEnd - nAtStart;
	QscUpdate(Dest, nSam);
}

// Run a section of one frame,
// e.g. 0x200 to 0x400 for second half
inline static void SectRun(int nStart,int nEnd)
{
	int nAtEnd, nAtStart, nWant, nDid = 0;

	if (nEnd <= nStart) {
		return;
	}

	nAtStart = (nCpsZ80Cycles * nStart) >> 10;
	nAtEnd   = (nCpsZ80Cycles * nEnd  ) >> 10;

	// Cycles would we like to do this time to get to end point
	nWant = nAtEnd - nAtStart - nZExtra;

	if (nWant > 0) {
		nDid = ZetRun(nWant);
	}
	nZExtra = nDid - nWant;

	SectSound(nStart, nEnd);
}

int QsndSectRun(int nStart,int nEnd)
{
	int nTo;
	// See if an irq occurs before the end
	while (nStart + nIrqNext < nEnd) {
		nTo = nStart + nIrqNext;
		SectRun(nStart, nTo);
		ZetRaiseIrq(0xff);
		ZetRun(0);
		ZetLowerIrq();

		nIrqNext = nIrqPeriod;	// Next irq happens after period
		nStart = nTo;			// Done this part now
	}

	nTo = nEnd;
	SectRun(nStart, nTo);
	nIrqNext -= nTo - nStart;	// Next irq happens sooner

	return 0;
}
